﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using JsDrawControl;
using System.Drawing.Drawing2D;

namespace JsDraw
{
    //=========================================================================
    /// <summary>
    /// ページ設定を行う画面です。
    /// </summary>
    public partial class PageSettingForm : Form
    {
        #region 用紙種別構造体

        private struct PaperKindListItem
        {
            /// <summary>用紙種別</summary>
            public PaperKind PaperKind;

            /// <summary>用紙サイズ</summary>
            public Size PaperSize;

            /// <summary>用紙名前</summary>
            public string PaperName;

            /// <summary>
            /// コンストラクタ
            /// </summary>
            /// <param name="paperKind"></param>
            /// <param name="paperSize"></param>
            /// <param name="paperName"></param>
            public PaperKindListItem(PaperKind paperKind, Size paperSize, string paperName)
            {
                this.PaperKind = paperKind;
                this.PaperSize = paperSize;
                this.PaperName = paperName;
            }

            /// <summary>
            /// 文字列表示
            /// </summary>
            /// <returns></returns>
            public override string ToString()
            {
                return this.PaperName;
            }
        }

        #endregion

        #region メンバ変数設定

        /// <summary>用紙種別</summary>
        protected PaperKind _paperKind;

        /// <summary>用紙向き</summary>
        protected PaperOrientation _paperOrientation;

        /// <summary>用紙サイズ</summary>
        protected SizeF _paperSize;

        /// <summary>列数</summary>
        protected int _colCount;

        /// <summary>行数</summary>
        protected int _rowCount;

        /// <summary>マージン</summary>
        protected bool _margin;

        /// <summary>マージンサイズ</summary>
        protected MarginsF _marginSize;

        /// <summary>ロードフラグ</summary>
        protected bool _loadFlag = false;

        /// <summary>プレビュービットマップ</summary>
        protected Bitmap _previewBmp = null;

        #endregion

        #region コンストラクタ、デストラクタ

        //=====================================================================
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public PageSettingForm()
        {
            InitializeComponent();

            // メンバ変数設定
            _paperKind = JsDrawControl.PaperKind.Custom;
            _paperOrientation = JsDrawControl.PaperOrientation.Portrait;
            _paperSize = new SizeF(0, 0);
            _colCount = 1;
            _rowCount = 1;
            _margin = false;
            _marginSize = new MarginsF(0, 0, 0, 0);
            _loadFlag = false;
            _previewBmp = new Bitmap(
                this.panelPreview.Width,
                this.panelPreview.Height,
                System.Drawing.Imaging.PixelFormat.Format32bppArgb);
        }

        #endregion

        #region パブリックプロパティ（用紙設定）

        //=====================================================================
        /// <summary>
        /// 用紙種別プロパティ
        /// </summary>
        public virtual PaperKind PaperKind
        {
            get { return _paperKind; }
            set
            {
                if (_paperKind != value)
                {
                    _paperKind = value;
                }
            }
        }

        //=====================================================================
        /// <summary>
        /// 用紙向きプロパティ
        /// </summary>
        public virtual PaperOrientation PaperOrientation
        {
            get { return _paperOrientation; }
            set
            {
                if (_paperOrientation != value)
                {
                    _paperOrientation = value;
                }
            }
        }

        //=====================================================================
        /// <summary>
        /// 用紙サイズプロパティ
        /// </summary>
        public virtual SizeF PaperSize
        {
            get { return _paperSize; }
            set
            {
                if (_paperSize != value)
                {
                    _paperSize = value;
                }
            }
        }

        //=====================================================================
        /// <summary>
        /// 列数プロパティ
        /// </summary>
        public virtual int ColCount
        {
            get { return _colCount; }
            set
            {
                if (_colCount != value)
                {
                    _colCount = value;
                }
            }
        } 

        //=====================================================================
        /// <summary>
        /// 行数プロパティ
        /// </summary>
        public virtual int RowCount
        {
            get { return _rowCount; }
            set
            {
                if (_rowCount != value)
                {
                    _rowCount = value;
                }
            }
        }

        //=====================================================================
        /// <summary>
        /// マージンプロパティ
        /// </summary>
        public virtual bool PaperMargin
        {
            get { return _margin; }
            set
            {
                if (_margin != value)
                {
                    _margin = value;
                }
            }
        }

        //=====================================================================
        /// <summary>
        /// マージンサイズプロパティ
        /// </summary>
        public virtual MarginsF MarginSize
        {
            get { return _marginSize; }
            set
            {
                if (!_marginSize.Equals(value))
                {
                    _marginSize = value;
                }
            }
        }
        
        #endregion

        #region フォームロードイベント処理

        //=====================================================================
        /// <summary>
        /// フォームロードイベントを処理します
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PageSettingForm_Load(object sender, EventArgs e)
        {
            #region     用紙種別コンボボックス設定
            this.comboPaperKind.Items.Clear();
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.Custom, Size.Empty, "カスタムサイズ"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.JapanesePostCard, new Size(107, 154), "官製はがき     (107㎜ × 154㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.JapanesePostCardR, new Size(214, 154), "往復はがき     (214㎜ × 154㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.PhotoE, new Size(82, 120), "E判            (82㎜ × 120㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.PhotoL, new Size(89, 127), "L判            (89㎜ × 127㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.PhotoHV, new Size(89, 152), "HV判           (89㎜ × 152㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.PhotoLP, new Size(89, 254), "LP判           (89㎜ × 254㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.PhotoKC, new Size(100, 148), "KC判           (100㎜ × 148㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.PhotoKH, new Size(102, 178), "KH判           (102㎜ × 178㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.PhotoKP, new Size(102, 305), "KP判           (102㎜ × 305㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.Photo2L, new Size(127, 178), "2L判           (127㎜ × 178㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.Photo2LH, new Size(127, 226), "2LH判          (127㎜ × 226㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.Photo2LP, new Size(127, 365), "2LP判          (127㎜ × 365㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.Letter, new Size(216, 280), "便箋           (216㎜ × 280㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.Legal, new Size(216, 356), "公文書         (216㎜ × 356㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.Tabloid, new Size(280, 432), "夕刊紙         (280㎜ × 432㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.CompactDisc, new Size(120, 120), "CDケース       (120㎜ × 120㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.A0, new Size(841, 1189), "A0             (841㎜ × 1189㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.A1, new Size(594, 841), "A1             (594㎜ × 841㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.A2, new Size(420, 594), "A2             (420㎜ × 594㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.A3, new Size(297, 420), "A3             (297㎜ × 420㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.A4, new Size(210, 297), "A4             (210㎜ × 297㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.A5, new Size(148, 210), "A5             (148㎜ × 210㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.A6, new Size(105, 148), "A6             (105㎜ × 148㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.A7, new Size(74, 105), "A7             (74㎜ × 105㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.A8, new Size(52, 74), "A8             (52㎜ ×74㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.A9, new Size(37, 52), "A9             (37㎜ ×52㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.A10, new Size(26, 37), "A10            (26㎜ × 37㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.B0, new Size(1030, 1456), "B0             (1030㎜ × 1456㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.B1, new Size(728, 1030), "B3             (728㎜ × 1030㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.B2, new Size(515, 728), "B3             (515㎜ × 728㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.B3, new Size(364, 515), "B3             (364㎜ × 515㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.B4, new Size(257, 364), "B4             (257㎜ × 364㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.B5, new Size(182, 257), "B5             (182㎜ × 257㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.B6, new Size(128, 182), "B6             (128㎜ × 182㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.B7, new Size(91, 128), "B7             (91㎜ × 128㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.B8, new Size(64, 91), "B8             (64㎜ × 91㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.B9, new Size(45, 64), "B9             (45㎜ × 64㎜)"));
            this.comboPaperKind.Items.Add(
                new PaperKindListItem(PaperKind.B10, new Size(32, 45), "B10            (32㎜ × 45㎜)"));

            #endregion

            // ========== 画面設定 ==========
            // 用紙種別
            foreach (PaperKindListItem item in this.comboPaperKind.Items)
            {
                if (item.PaperKind == _paperKind)
                {
                    this.comboPaperKind.SelectedItem = item;
                    break;
                }
            }

            // 用紙向き
            if (_paperOrientation == JsDrawControl.PaperOrientation.Portrait)
            {
                this.radioPortrait.Checked = true;
            }
            else
            {
                this.radioLandscape.Checked = true;
            }

            // 用紙サイズ
            if (_paperKind == JsDrawControl.PaperKind.Custom)
            {
                this.updSizeWidth.Enabled = true;
                this.updSizeHeight.Enabled = true;
            }
            else
            {
                this.updSizeWidth.Enabled = false;
                this.updSizeHeight.Enabled = false;
            }
            this.updSizeWidth.Value = (decimal)_paperSize.Width;
            this.updSizeHeight.Value = (decimal)_paperSize.Height;

            // ページ数
            this.updColCount.Value = (decimal)_colCount;
            this.updRowCount.Value = (decimal)_rowCount;

            // 余白
            this.checkMargin.Checked = _margin;
            this.updMarginTop.Enabled = _margin;
            this.updMarginBottom.Enabled = _margin;
            this.updMarginLeft.Enabled = _margin;
            this.updMarginRight.Enabled = _margin;

            this.updMarginTop.Value = (decimal)_marginSize.Top;
            this.updMarginBottom.Value = (decimal)_marginSize.Bottom;
            this.updMarginLeft.Value = (decimal)_marginSize.Left;
            this.updMarginRight.Value = (decimal)_marginSize.Right;

            // プレビュー描画
            this.DrawPreview();

            // ロードフラグ設定
            _loadFlag = true;
        }

        #endregion

        #region 用紙種類

        //=====================================================================
        /// <summary>
        /// 用紙種類コンボボックスの変更イベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void comboPaperKind_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (_loadFlag == false)
                return;

            // サイズ設定
            PaperKindListItem item = (PaperKindListItem)this.comboPaperKind.SelectedItem;

            try
            {
                _loadFlag = false;

                // ページサイズ設定
                if (item.PaperKind == JsDrawControl.PaperKind.Custom)
                {
                    this.updSizeWidth.Enabled = true;
                    this.updSizeHeight.Enabled = true;

                    this.updSizeWidth.Value = (decimal)_paperSize.Width;
                    this.updSizeHeight.Value = (decimal)_paperSize.Height;
                }
                else
                {
                    this.updSizeWidth.Enabled = false;
                    this.updSizeHeight.Enabled = false;

                    this.updSizeWidth.Value = item.PaperSize.Width;
                    this.updSizeHeight.Value = item.PaperSize.Height;
                }

                // マージンサイズチェック
                this.CheckMarginSize();

                // プレビュー描画
                this.DrawPreview();
                this.panelPreview.Refresh();
            }
            finally
            {
                _loadFlag = true;
            }
        }

        #endregion

        #region 用紙向き

        //=====================================================================
        /// <summary>
        /// 向きラジオボタン変更イベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void radioPaperOrientation_CheckedChanged(object sender, EventArgs e)
        {
            if (_loadFlag == false)
                return;

            try
            {
                _loadFlag = false;

                // マージンサイズチェック
                this.CheckMarginSize();

                // プレビュー描画
                this.DrawPreview();
                this.panelPreview.Refresh();
            }
            finally
            {
                _loadFlag = true;
            }
        }

        #endregion

        #region 用紙サイズ

        //=====================================================================
        /// <summary>
        /// 用紙サイズのUpDownボックスの変更イベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void updSize_ValueChanged(object sender, EventArgs e)
        {
            if (_loadFlag == false)
                return;

            try
            {
                _loadFlag = false;

                // マージンサイズチェック
                this.CheckMarginSize();

                // プレビュー描画
                this.DrawPreview();
                this.panelPreview.Refresh();
            }
            finally
            {
                _loadFlag = true;
            }
        }

        #endregion

        #region ページ数

        //=====================================================================
        /// <summary>
        /// ページ数のUpDownボックスの変更イベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void updPageCount_ValueChanged(object sender, EventArgs e)
        {
            if (_loadFlag == false)
                return;

            // プレビュー描画
            this.DrawPreview();
            this.panelPreview.Refresh();
        }

        #endregion

        #region 余白

        //=====================================================================
        /// <summary>
        /// 「余白あり」チェックボックスの変更イベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void checkMargin_CheckedChanged(object sender, EventArgs e)
        {
            if (_loadFlag == false)
                return;

            try
            {
                _loadFlag = false;

                // 余白
                this.checkMargin.Checked = this.checkMargin.Checked;
                this.updMarginTop.Enabled = this.checkMargin.Checked;
                this.updMarginBottom.Enabled = this.checkMargin.Checked;
                this.updMarginLeft.Enabled = this.checkMargin.Checked;
                this.updMarginRight.Enabled = this.checkMargin.Checked;

                // マージンサイズチェック
                this.CheckMarginSize();

                // プレビュー描画
                this.DrawPreview();
                this.panelPreview.Refresh();
            }
            finally
            {
                _loadFlag = true;
            }
        }

        //=====================================================================
        /// <summary>
        /// 余白「上」のUpDownボックスの変更イベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void updMarginTop_ValueChanged(object sender, EventArgs e)
        {
            if (_loadFlag == false)
                return;

            try
            {
                _loadFlag = false;

                // マージンサイズチェック
                if (this.radioPortrait.Checked)
                {
                    if ((this.updSizeHeight.Value) < (this.updMarginTop.Value + this.updMarginBottom.Value))
                        this.updMarginTop.Value = (this.updSizeHeight.Value - this.updMarginBottom.Value);
                }
                else
                {
                    if ((this.updSizeWidth.Value) < (this.updMarginTop.Value + this.updMarginBottom.Value))
                        this.updMarginTop.Value = this.updSizeWidth.Value - this.updMarginBottom.Value;
                }

                // プレビュー描画
                this.DrawPreview();
                this.panelPreview.Refresh();
            }
            finally
            {
                _loadFlag = true;
            }
        }

        //=====================================================================
        /// <summary>
        /// 余白「下」のUpDownボックスの変更イベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void updMarginBottom_ValueChanged(object sender, EventArgs e)
        {
            if (_loadFlag == false)
                return;

            try
            {
                _loadFlag = false;

                // マージンサイズチェック
                if (this.radioPortrait.Checked)
                {
                    if ((this.updSizeHeight.Value) < (this.updMarginTop.Value + this.updMarginBottom.Value))
                        this.updMarginBottom.Value = (this.updSizeHeight.Value - this.updMarginTop.Value);
                }
                else
                {
                    if ((this.updSizeWidth.Value) < (this.updMarginTop.Value + this.updMarginBottom.Value))
                        this.updMarginBottom.Value = this.updSizeWidth.Value - this.updMarginTop.Value;
                }

                // プレビュー描画
                this.DrawPreview();
                this.panelPreview.Refresh();
            }
            finally
            {
                _loadFlag = true;
            }
        }

        //=====================================================================
        /// <summary>
        /// 余白「左」のUpDownボックスの変更イベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void updMarginLeft_ValueChanged(object sender, EventArgs e)
        {
            if (_loadFlag == false)
                return;

            try
            {
                _loadFlag = false;

                // マージンサイズチェック
                if (this.radioPortrait.Checked)
                {
                    if ((this.updSizeWidth.Value) < (this.updMarginRight.Value + this.updMarginLeft.Value))
                        this.updMarginLeft.Value = (this.updSizeWidth.Value - this.updMarginRight.Value);
                }
                else
                {
                    if ((this.updSizeHeight.Value) < (this.updMarginRight.Value + this.updMarginLeft.Value))
                        this.updMarginLeft.Value = this.updSizeHeight.Value - this.updMarginRight.Value;
                }

                // プレビュー描画
                this.DrawPreview();
                this.panelPreview.Refresh();
            }
            finally
            {
                _loadFlag = true;
            }
        }

        //=====================================================================
        /// <summary>
        /// 余白「右」のUpDownボックスの変更イベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void updMarginRight_ValueChanged(object sender, EventArgs e)
        {
            if (_loadFlag == false)
                return;

            try
            {
                _loadFlag = false;

                // マージンサイズチェック
                if (this.radioPortrait.Checked)
                {
                    if ((this.updSizeWidth.Value) < (this.updMarginRight.Value + this.updMarginLeft.Value))
                        this.updMarginRight.Value = (this.updSizeWidth.Value - this.updMarginLeft.Value);
                }
                else
                {
                    if ((this.updSizeHeight.Value) < (this.updMarginRight.Value + this.updMarginLeft.Value))
                        this.updMarginRight.Value = this.updSizeHeight.Value - this.updMarginLeft.Value;
                }

                // プレビュー描画
                this.DrawPreview();
                this.panelPreview.Refresh();
            }
            finally
            {
                _loadFlag = true;
            }
        }

        #endregion

        #region 「OK」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「OK」ボタンクリックイベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonOk_Click(object sender, EventArgs e)
        {
            // 用紙種別
            PaperKindListItem item = (PaperKindListItem)this.comboPaperKind.SelectedItem;
            _paperKind = item.PaperKind;


            // 用紙向き
            if (this.radioLandscape.Checked)
            {
                _paperOrientation = JsDrawControl.PaperOrientation.LandScape;
            }
            else
            {
                _paperOrientation = JsDrawControl.PaperOrientation.Portrait;
            }

            // 用紙サイズ
            if (_paperKind == JsDrawControl.PaperKind.Custom)
            {
                _paperSize.Width = (int)this.updSizeWidth.Value;
                _paperSize.Height = (int)this.updSizeHeight.Value;
            }

            // ページ数
            _colCount = (int)this.updColCount.Value;
            _rowCount = (int)this.updRowCount.Value;

            // 余白
            _margin = this.checkMargin.Checked;
            if( _margin){
                _marginSize.Top = (int)this.updMarginTop.Value;
                _marginSize.Bottom = (int)this.updMarginBottom.Value;
                _marginSize.Left = (int)this.updMarginLeft.Value;
                _marginSize.Right = (int)this.updMarginRight.Value;
            }

            // 画面を閉じる（OK）
            this.DialogResult = System.Windows.Forms.DialogResult.OK;
            this.Close();
        }

        #endregion

        #region 「キャンセル」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「キャンセル」ボタンクリックイベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonCancel_Click(object sender, EventArgs e)
        {
            // 画面を閉じる（Cancel）
            this.DialogResult = System.Windows.Forms.DialogResult.Cancel;
            this.Close();
        }

        #endregion

        #region 余白チェック

        //=====================================================================
        /// <summary>
        /// マージンサイズをチェックします。
        /// </summary>
        private void CheckMarginSize()
        {
            // マージンチェック
            if (this.checkMargin.Checked)
            {
                bool errFlag = false;

                // 用紙の向きを考慮する。
                if (this.radioPortrait.Checked)
                {
                    // 幅
                    if ((this.updSizeWidth.Value) < (this.updMarginLeft.Value + this.updMarginRight.Value))
                        errFlag = true;
                    // 高さ
                    if ((this.updSizeHeight.Value) < (this.updMarginTop.Value + this.updMarginBottom.Value))
                        errFlag = true;
                }
                else
                {
                    // 高さ
                    if ((this.updSizeHeight.Value) < (this.updMarginLeft.Value + this.updMarginRight.Value))
                        errFlag = true;
                    // 幅
                    if ((this.updSizeWidth.Value) < (this.updMarginTop.Value + this.updMarginBottom.Value))
                        errFlag = true;
                }

                if (errFlag)
                {
                    MessageBox.Show(this, "余白のサイズが不正なため設定し直してください。",
                        Program.PROGRAM_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Warning);

                    // 余白リセット
                    this.updMarginTop.Value = 0;
                    this.updMarginBottom.Value = 0;
                    this.updMarginLeft.Value = 0;
                    this.updMarginRight.Value = 0;
                }
            }
        }
        
        #endregion

        #region レビュー描画

        //=====================================================================
        /// <summary>
        /// プレビューを描画します。
        /// </summary>
        private void DrawPreview()
        {
            int w = this.radioPortrait.Checked ? (int)this.updSizeWidth.Value : (int)this.updSizeHeight.Value;
            int h = this.radioPortrait.Checked ? (int)this.updSizeHeight.Value : (int)this.updSizeWidth.Value;

            // マージンサイズ
            Margins margin = new Margins(
                this.checkMargin.Checked ? (int)this.updMarginLeft.Value : 0,
                this.checkMargin.Checked ? (int)this.updMarginTop.Value : 0,
                this.checkMargin.Checked ? (int)this.updMarginRight.Value : 0,
                this.checkMargin.Checked ? (int)this.updMarginBottom.Value : 0);

            // 余白を除いたページサイズ計算
            Size pageSize = new Size(
                w - (margin.Left + margin.Right),
                h - (margin.Top + margin.Bottom));

            // 余白を除いた内側のサイズ
            Size innerSize = new Size(
                pageSize.Width * (int)this.updColCount.Value,
                pageSize.Height * (int)this.updRowCount.Value);

            // 余白を含んだ外側のサイズ
            Size outerSize = new Size(
                innerSize.Width + margin.Left + margin.Right,
                innerSize.Height + margin.Top + margin.Bottom);

            // 用紙の描画範囲
            Rectangle drawBounds = new Rectangle(60, 30, 240, 270);

            // ========== 描画する矩形を計算 ==========
            float outerRange = (float)outerSize.Width / (float)outerSize.Height;
            float drawReange = (float)drawBounds.Width / (float)drawBounds.Height;

            Rectangle drawOuterRect = drawBounds;
            Rectangle drawInnerRect = drawBounds;

            // 余白を含んだ外側の矩形サイズ
            if (drawReange < outerRange)
            {
                // 横に合わせる
                drawOuterRect.Height = (int)(((float)drawBounds.Width / (float)outerSize.Width) * (float)outerSize.Height);
                drawOuterRect.Y = drawBounds.Y + ((drawBounds.Height - drawOuterRect.Height) / 2);
            }
            else
            {
                // 縦に合わせる
                drawOuterRect.Width = (int)(((float)drawBounds.Height / (float)outerSize.Height) * (float)outerSize.Width);
                drawOuterRect.X = drawBounds.X + ((drawBounds.Width - drawOuterRect.Width) / 2);
            }

            // 余白を除いた外側の矩形サイズ
            if (this.checkMargin.Checked)
            {
                Margins drawMargin = new Margins(
                    (int)(((decimal)drawOuterRect.Width / (decimal)outerSize.Width) * this.updMarginLeft.Value),
                    (int)(((decimal)drawOuterRect.Height / (decimal)outerSize.Height) * this.updMarginTop.Value),
                    (int)(((decimal)drawOuterRect.Width / (decimal)outerSize.Width) * this.updMarginRight.Value),
                    (int)(((decimal)drawOuterRect.Height / (decimal)outerSize.Height) * this.updMarginBottom.Value));

                drawInnerRect = new Rectangle(
                    drawOuterRect.X + drawMargin.Left,
                    drawOuterRect.Y + drawMargin.Top,
                    drawOuterRect.Width - (drawMargin.Left + drawMargin.Right),
                    drawOuterRect.Height - (drawMargin.Top + drawMargin.Bottom));
            }
            else
            {
                drawInnerRect = drawOuterRect;
            }

            // ========== プレビュー描画 ==========
            lock (_previewBmp)
            {
                using( Graphics g = Graphics.FromImage(_previewBmp))
                {
                    g.Clear(Color.White);

                    // クオリティ設定
                    g.PageUnit = GraphicsUnit.Pixel;
                    g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.None;

                    // サイズを描画
                    StringFormat sf = new StringFormat(){
                        Alignment = StringAlignment.Center,
                        LineAlignment = StringAlignment.Center};

                    Pen aPen = new Pen(Color.Black, 1);
                    aPen.StartCap = LineCap.ArrowAnchor;
                    aPen.EndCap = LineCap.ArrowAnchor;

                    // 幅の描画
                    g.DrawLine(Pens.Black,
                        drawOuterRect.Left, drawOuterRect.Top - 12,
                        drawOuterRect.Left, drawOuterRect.Top - 2);
                    g.DrawLine(Pens.Black,
                        drawOuterRect.Right, drawOuterRect.Top - 12,
                        drawOuterRect.Right, drawOuterRect.Top - 2);
                    g.DrawLine(aPen,
                        drawOuterRect.Left + 2, drawOuterRect.Top - 7,
                        drawOuterRect.Right - 2, drawOuterRect.Top - 7);

                    g.DrawString(string.Format("{0}㎜", outerSize.Width), this.Font, Brushes.Black,
                        new Rectangle(
                            drawBounds.Left, drawOuterRect.Top - 25,
                            drawBounds.Width, 15), sf);

                    // 高さの描画
                    g.DrawLine(Pens.Black,
                        drawOuterRect.Left - 12, drawOuterRect.Top,
                        drawOuterRect.Left - 2, drawOuterRect.Top);
                    g.DrawLine(Pens.Black,
                        drawOuterRect.Left - 12, drawOuterRect.Bottom,
                        drawOuterRect.Left -2, drawOuterRect.Bottom);
                    g.DrawLine(aPen,
                        drawOuterRect.Left - 7, drawOuterRect.Top + 2,
                        drawOuterRect.Left - 7, drawOuterRect.Bottom - 2);

                    g.DrawString(string.Format("{0}㎜", outerSize.Height), this.Font, Brushes.Black,
                        new Rectangle(
                            drawOuterRect.Left - 55, 
                            drawBounds.Top, 
                            55, 
                            drawBounds.Height), sf);

                    // 影
                    g.FillRectangle(Brushes.LightGray,
                        new Rectangle(
                            drawOuterRect.X + 5,
                            drawOuterRect.Y + 5,
                            drawOuterRect.Width,
                            drawOuterRect.Height));

                    // 背景
                    g.FillRectangle(Brushes.White,drawOuterRect);

                    // ふち
                    g.DrawRectangle(Pens.Black, drawOuterRect);

                    // 余白
                    if (this.checkMargin.Checked)
                    {
                        Pen mPen = new Pen(Color.Gray, 1);
                        mPen.DashStyle = DashStyle.Dot;
                        g.DrawRectangle(mPen, drawInnerRect);
                    }

                    // ページ区切り
                    if( 1 < this.updColCount.Value)
                    {
                        decimal pw = (decimal)drawInnerRect.Width / this.updColCount.Value;
                        for (int i = 1; i < this.updColCount.Value; i++)
                        {
                            int dx = (int)(drawInnerRect.X + (i * pw));
                            g.DrawLine(Pens.Gray, dx, drawOuterRect.Top, dx, drawOuterRect.Bottom);
                        }                    
                    }
                    if (1 < this.updRowCount.Value)
                    {
                        decimal pw = (decimal)drawInnerRect.Height / this.updRowCount.Value;
                        for (int i = 1; i < this.updRowCount.Value; i++)
                        {
                            int dy = (int)(drawInnerRect.Y + (i * pw));
                            g.DrawLine(Pens.Gray, drawOuterRect.Left, dy, drawOuterRect.Right, dy);
                        }
                    }
                }
            }
        }

        //=====================================================================
        /// <summary>
        /// プレビュー描画イベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void panelPreview_Paint(object sender, PaintEventArgs e)
        {
            if (_previewBmp != null)
            {
                lock (_previewBmp)
                {
                    e.Graphics.DrawImage(_previewBmp, 0, 0);
                }
            }
        }

        #endregion

    }
}
